# aemo_to_tariff/powercor.py
from datetime import datetime, timedelta
from zoneinfo import ZoneInfo
from datetime import time

def time_zone():
    return 'Australia/Melbourne'


tariffs = {
    'D1': {
        'name': 'Residential Single Rate',
        'periods': [
            ('Anytime', time(0, 0), time(23, 59), 9.6700)
        ]
    },
    'PRTOU': {
        'name': 'Residential TOU',
        'periods': [
            ('Off-peak', time(0, 0), time(15, 0), 5.07),
            ('Peak', time(15, 0), time(21, 0), 20.17),
            ('Off-peak', time(21, 0), time(23, 59), 5.07),
        ]
    },
    'NDMO21': {
        'name': 'NDMO21 TOU',
        'periods': [
            ('Off-peak', time(0, 0), time(15, 0), 5.61),
            ('Peak', time(15, 0), time(21, 0), 19.29),
            ('Off-peak', time(21, 0), time(23, 59), 5.61),
        ]
    },
    'NDTOU': {
        'name': 'NDTOU TOU',
        'periods': [
            ('Off-peak', time(0, 0), time(15, 0), 4.58),
            ('Peak', time(15, 0), time(21, 0), 20.60),
            ('Off-peak', time(21, 0), time(23, 59), 4.58),
        ]
    },
    'PRDS': {
        'name': 'Residential daytime saver',
        'periods': [
            ('Off-peak', time(0, 0), time(10, 0), 7.0),
            ('Day', time(10, 0), time(15, 0), 0.0),
            ('Off-peak', time(15, 0), time(16, 00), 7.0),
            ('Peak', time(16, 0), time(21, 0), 19.61),
            ('Off-peak', time(21, 0), time(23, 59), 7.0),
        ]
    }
}

demand_charges = {
    'PRDEMD': { 'Peak': 10.0},  # Residential Demand
    'PSTDEMD': { 'Peak': 10.0},  # Residential Demand
    'PSTCTD': { 'Peak': 10.0},  # Residential Demand
    'PSTNPD': { 'Peak': 10.0},  # Residential Demand
    'PSTNCD': { 'Peak': 10.0},  # Residential Demand
    'PSTNDD': { 'Peak': 10.0},  # Residential Demand
}

def get_daily_fee(tariff_code: str):
    return 41.10

def get_periods(tariff_code: str):
    tariff = tariffs.get(tariff_code)
    if not tariff:
        raise ValueError(f"Unknown tariff code: {tariff_code}")

    return tariff['periods']

def convert_feed_in_tariff(interval_datetime: datetime, tariff_code: str, rrp: float):
    """
    Convert RRP from $/MWh to c/kWh for SA Power Networks.

    Parameters:
    - interval_datetime (datetime): The interval datetime.
    - tariff_code (str): The tariff code.
    - rrp (float): The Regional Reference Price in $/MWh.

    Returns:
    - float: The price in c/kWh.
    """
    rrp_c_kwh = rrp / 10
    
    return rrp_c_kwh

def convert(interval_datetime: datetime, tariff_code: str, rrp: float):
    """
    Convert RRP from $/MWh to c/kWh for Powercor.

    Parameters:
    - interval_time (str): The interval time.
    - tariff (str): The tariff code.
    - rrp (float): The Regional Reference Price in $/MWh.

    Returns:
    - float: The price in c/kWh.
    """
    interval_datetime = interval_datetime - timedelta(minutes=5)
    interval_time = interval_datetime.astimezone(ZoneInfo(time_zone())).time()

    rrp_c_kwh = rrp / 10
    tariff = tariffs[tariff_code]

    # Find the applicable period and rate
    for period, start, end, rate in tariff['periods']:
        if start <= interval_time < end:
            total_price = rrp_c_kwh + rate
            return total_price

        # Handle overnight periods (e.g., 22:00 to 07:00)
        if start > end and (interval_time >= start or interval_time < end):
            total_price = rrp_c_kwh + rate
            return total_price

    # Otherwise, this terrible approximation
    slope = 1.037869032618134
    intercept = 5.586606750833143
    return rrp_c_kwh * slope + intercept


def estimate_demand_fee(interval_time: datetime, tariff_code: str, demand_kw: float):
    """
    Estimate the demand fee for a given tariff code, demand amount, and time period.

    Parameters:
    - interval_time (datetime): The interval datetime.
    - tariff_code (str): The tariff code.
    - demand_kw (float): The maximum demand in kW (or kVA for 8100 and 8300 tariffs).

    Returns:
    - float: The estimated demand fee in dollars.
    """
    time_of_day = interval_time.astimezone(ZoneInfo(time_zone())).time()
    
    if tariff_code not in demand_charges:
        return 0.0  # Return 0 if the tariff doesn't have a demand charge

    charge = demand_charges[tariff_code]
    if isinstance(charge, dict):
        # Determine the time period
        if 'Peak' in charge and time(17, 0) <= time_of_day < time(20, 0):
            charge_per_kw_per_month = charge['Peak']
        elif 'Off-Peak' in charge and time(11, 0) <= time_of_day < time(13, 0):
            charge_per_kw_per_month = charge['Off-Peak']
        else:
            charge_per_kw_per_month = charge.get('Shoulder', 0.0)
    else:
        charge_per_kw_per_month = charge

    return charge_per_kw_per_month * demand_kw


import unittest
from datetime import datetime
from zoneinfo import ZoneInfo
import aemo_to_tariff.essential as essential
from aemo_to_tariff.essential import time_zone, convert_feed_in_tariff, convert

class TestEssentualPower(unittest.TestCase):
    def test_some_essential_functionality(self):
        interval_time = datetime(2025, 2, 20, 9, 10, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLNT3AL'
        rrp = -100.0
        expected_price = 3.858878319999999
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.168, expected_price, places=1)

    def test_seven_pm(self):
        # 2025-05-12 19:25:00+10
        interval_time = datetime(2025, 5, 12, 19, 25, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLND1AR'
        rrp = 135.36
        expected_price = 37.60955297
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.68, expected_price, places=1)

    def test_seven_am(self):
        # 2025-05-12 07:25:00+10
        interval_time = datetime(2025, 5, 12, 7, 25, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLND1AR'
        rrp = 153.88
        expected_price = 34.06917696
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.6, expected_price, places=1)

    def test_four_am(self):
        # 2025-05-10 04:15:00+10
        interval_time = datetime(2025, 5, 20, 4, 15, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLND1AR'
        rrp = 67.73
        expected_price = 15.23709699
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.48, expected_price, places=1)

    def test_later_day(self):
        # 2025-05-10 12:25:00+10
        interval_time = datetime(2025, 5, 20, 12, 25, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLND1AR'
        rrp = -16.13
        expected_price = 6.21313241
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.447, expected_price, places=1)

    def test_solar_soaker(self):
        # 
        # 00:50	$80	Act	0.002 x 7.48¢	1.49¢	0 x 16.17¢	
        interval_time = datetime(2025, 11, 9, 00, 15, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'BLNRSS2'
        export_tariff = 'BLNREX2'
        feed_in_price = convert_feed_in_tariff(interval_time, export_tariff, 80.0)
        msg = f"Feed-in price for {export_tariff} at {interval_time} should be approximately 7.48"
        self.assertAlmostEqual(feed_in_price, 7.48 + 0.52, places=1, msg=msg)
        buyer_price = convert(interval_time, tariff_code, 80.0)
        msg = f"Buyer price for {tariff_code} at {interval_time} should be approximately 16.17"
        self.assertAlmostEqual(buyer_price, 16.17 - 2.32, places=1, msg=msg)
        
        # 07:50	$-10	Act	0.001 x -0.94¢	-0.09¢	0 x 19.13¢
        interval_time = datetime(2025, 11, 9, 7, 50, tzinfo=ZoneInfo(time_zone()))
        feed_in_price = convert_feed_in_tariff(interval_time, export_tariff, -10.0)
        msg = f"Feed-in price for {export_tariff} at {interval_time} should be approximately -0.09"
        self.assertAlmostEqual(feed_in_price, -0.09 - 0.91, places=1, msg=msg)
        buyer_price = convert(interval_time, tariff_code, -10.0)
        msg = f"Buyer price for {tariff_code} at {interval_time} should be approximately 19.13"
        self.assertAlmostEqual(buyer_price, 19.13 - 3.18, places=1, msg=msg)

        # 11:55	$-26	Act	0.001 x -11.43¢	-1.14¢	0 x 5.26¢
        interval_time = datetime(2025, 11, 9, 11, 55, tzinfo=ZoneInfo(time_zone()))
        feed_in_price = convert_feed_in_tariff(interval_time, export_tariff, -26.0)
        msg = f"Feed-in price for {export_tariff} at {interval_time} should be approximately -2.6"
        self.assertAlmostEqual(feed_in_price, -3.42, places=1, msg=msg)
        buyer_price = convert(interval_time, tariff_code, -26.0)
        msg = f"Buyer price for {tariff_code} at {interval_time} should be approximately 5.26"
        self.assertAlmostEqual(buyer_price, 5.26 - 2, places=1, msg=msg)
        
        # 18:10	$84	Act	0.001 x 19.45¢	1.94¢	0.001 x 28.82¢
        interval_time = datetime(2025, 11, 9, 18, 10, tzinfo=ZoneInfo(time_zone()))
        feed_in_price = convert_feed_in_tariff(interval_time, export_tariff, 84.0)
        msg = f"Feed-in price for {export_tariff} at {interval_time} should be approximately 19.45"
        self.assertAlmostEqual(feed_in_price, 19.45 + 0.53, places=1, msg=msg)
        buyer_price = convert(interval_time, tariff_code, 84.0)
        msg = f"Buyer price for {tariff_code} at {interval_time} should be approximately 28.82"
        self.assertAlmostEqual(buyer_price, 25.35, places=1, msg=msg)




# test/test_convert.py
import unittest
from datetime import datetime
from aemo_to_tariff import (spot_to_tariff, get_daily_fee, calculate_demand_fee,
                            spot_to_feed_in_tariff, estimate_demand_fee, battery_tariffs)

class TestTariffConversions(unittest.TestCase):

    def test_battery_tariff(self):
        expected_includes = 'BLNREX2'
        self.assertIn(expected_includes, battery_tariffs(network='Essential', customer_type='Residential')['export'])
        expected_includes = 'EA029'
        self.assertIn(expected_includes, battery_tariffs(network='Ausgrid', customer_type='Residential')['export'])
        expected_includes = 'RESELE'
        self.assertIn(expected_includes, battery_tariffs(network='SAPN', customer_type='Residential')['export'])
        expected_includes = 'N61'
        self.assertIn(expected_includes, battery_tariffs(network='Endeavour', customer_type='Residential')['export'])
        
    def test_energex_tariff_6970(self):
        # Off peak
        interval_time = datetime.strptime('2024-07-05 14:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Energex', '6900', 100, 1, 1), 10.159, 2)

        # Peak
        interval_time = datetime.strptime('2024-07-05 18:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Energex', '6970', 100, 1, 1), 29.521, 2)

        # Shoulder
        interval_time = datetime.strptime('2024-07-05 02:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Energex', '6900', 100, 1, 1), 15.022, 2)

        # With loss factor
        interval_time = datetime.strptime('2024-07-05 14:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Energex', '6900', 200, 1.05, 1.01), 21.541, 2)
        
        # Demand estimate
        interval_time = datetime.strptime('2024-07-05 18:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Energex', '6900', 5.5), 0.0, 2)
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Energex', '3950', 5.5), 28.1985, 2)

    def test_powercor_tariff_017(self):
        # With loss factor 6.2516935 -44.75
        interval_time = datetime.strptime('2024-07-05 14:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Powercor', 'PRDS', -18.79, 1.058, 1.01), -2.08574, 1)

        # PRDS estimate
        interval_time = datetime.strptime('2024-09-05 15:05+10:00', '%Y-%m-%d %H:%M%z')
        expected_price = (17.95919 * 0.99) - 2.25 - 3.9606291203999966
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Powercor', 'PRDS', 42.53, 1.058, 1.00), expected_price, 2)
    
    def test_ergon_tariff_017(self):
        # With loss factor
        interval_time = datetime.strptime('2024-07-05 14:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Ergon', 'ERTOUET1', 200, 1.05, 1.01), 22.06, 2)
        
        # Demand estimate
        interval_time = datetime.strptime('2024-07-05 18:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Ergon', 'ERTOUET1', 5.5), 0.0, 2)
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Ergon', 'ERTDEMCT1', 5.5), 38.5, 2)

    def test_evoenergy_tariff_017(self):
        # Off peak
        interval_time = datetime.strptime('2024-07-05 14:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Evoenergy', '017', 100, 1, 1), 13.7411, 2)

        # Peak
        interval_time = datetime.strptime('2024-07-05 17:05+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Evoenergy', '017', 100, 1, 1), 27.9564, 2)

        # Shoulder
        interval_time = datetime.strptime('2024-07-05 02:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Evoenergy', '017', 100, 1, 1), 16.3855, 2)

        interval_time = datetime.strptime('2024-07-05 14:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Evoenergy', '017', 200), 25.4255, 2)

    def test_ausgrid_tariff_EA116(self):
        interval_time = datetime.strptime('2024-07-05 14:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Ausgrid', 'EA116', 100, 1, 1), 12.491, 2)
        interval_time = datetime.strptime('2024-07-05 14:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Ausgrid', 'EA116', 200, 1, 1), 22.645, 2)
        # Demand estimate
        interval_time = datetime.strptime('2024-07-05 18:00+11:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Ausgrid', 'EA025', 5.5), 0.0, 2)
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Ausgrid', 'EA116', 5.5), 49.489, 2)

    def test_essential_demand_fee(self):
        # Demand estimate
        interval_time = datetime.strptime('2024-07-05 18:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Essential', 'BLNRSS2', 5.5), 0.0, 2)
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Essential', 'Other', 5.5), 49.489, 2)

    def test_endeavour_demand_fee(self):
        # Demand estimate
        interval_time = datetime.strptime('2024-07-05 18:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Endeavour', 'N71', 5.5), 0.0, 2)
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Endeavour', 'N73', 5.5), 29.92, 2)
        interval_time = datetime.strptime('2024-07-05 11:30+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'Endeavour', 'N73', 5.5), 0.0, 2)

    def test_energex_daily_fee(self):
        self.assertAlmostEqual(get_daily_fee('Energex', '3900'), 0.556, 3)
        self.assertAlmostEqual(get_daily_fee('Energex', '7200'), 7.665, 3)
        self.assertAlmostEqual(get_daily_fee('Energex', '6000', annual_usage=15000), 0.739, 3)
        self.assertAlmostEqual(get_daily_fee('Energex', '6000', annual_usage=30000), 1.033, 3)

    def test_energex_demand_fee(self):
        self.assertAlmostEqual(calculate_demand_fee('Energex', '3700', 5.5, 31), 0.0, 2)
        self.assertAlmostEqual(calculate_demand_fee('Energex', '3900', 5.5, 31), 0.0, 2)

    def test_ausgrid_daily_fee(self):
        # Placeholder test - update when Ausgrid is implemented
        self.assertEqual(get_daily_fee('Ausgrid', 'EA116'), 0.6)

    def test_ausgrid_demand_fee(self):
        # Placeholder test - update when Ausgrid is implemented
        self.assertAlmostEqual(calculate_demand_fee('Ausgrid', 'EA116', 5.5, 31), 49.489, 1)

    def test_evoenergy_daily_fee(self):
        # Placeholder test - update when Evoenergy is implemented
        self.assertEqual(get_daily_fee('Evoenergy', '017'), 0.0)

    def test_evoenergy_demand_fee(self):
        # Placeholder test - update when Evoenergy is implemented
        self.assertEqual(calculate_demand_fee('Evoenergy', '017', 5.5, 31), 0.0)

    def test_sapn_daily_fee(self):
        self.assertAlmostEqual(get_daily_fee('SAPN', 'RTOU'), 64.4, 4)
        self.assertAlmostEqual(get_daily_fee('SAPN', 'SBTOU'), 72.59, 4)
        # Demand estimate
        interval_time = datetime.strptime('2024-07-05 18:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'SAPN', 'SBTOU', 5.5), 0.0, 2)
        self.assertAlmostEqual(estimate_demand_fee(interval_time, 'SAPN', 'RTOU', 5.5), 0, 2)

    def test_evo_battery_trial(self):
        interval_time = datetime.strptime('2024-07-05 14:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'Evoenergy', '026', 100), 14.506, 2)
        self.assertAlmostEqual(spot_to_feed_in_tariff(interval_time, 'Evoenergy', '026', 200), 20.84, 2)
        interval_time = datetime.strptime('2024-07-05 19:00+10:00', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_feed_in_tariff(interval_time, 'Evoenergy', '026', 200), 34.2, 2)

    def test_sapn_demand_fee(self):
        self.assertAlmostEqual(calculate_demand_fee('SAPN', 'RTOU', 5.5, 31), 0, 4)
        self.assertAlmostEqual(calculate_demand_fee('SAPN', 'SBTOU', 5.5, 31), 0, 4)

    def test_sapn_tariff_RTOU(self):
        # Peak
        interval_time = datetime.strptime('2024-07-05 18:00+09:30', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'SAPN', 'RTOU', 100), 29.8692, 2)

        # Off-peak
        interval_time = datetime.strptime('2024-07-05 02:00+09:30', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'SAPN', 'RTOU', 100), 20.3892, 2)

    def test_tasnetworks_daily_fee(self):
        self.assertAlmostEqual(get_daily_fee('tasnetworks', 'TAS93'), 70.032, 2)
        self.assertAlmostEqual(get_daily_fee('tasnetworks', 'TAS94'), 83.78, 2)

    def test_tasnetworks_demand_fee(self):
        self.assertAlmostEqual(calculate_demand_fee('tasnetworks', '75', 5.5, 31), 0.0, 2)
        self.assertAlmostEqual(calculate_demand_fee('tasnetworks', '31', 5.5, 31), 0.0, 2)

    def test_tasnetworks_tariff_93(self):
        # Peak
        interval_time = datetime.strptime('2024-07-05 18:00+09:30', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'tasnetworks', 'TAS93', 100), 28.148, 2)

        # Off-peak
        interval_time = datetime.strptime('2024-07-05 02:00+09:30', '%Y-%m-%d %H:%M%z')
        self.assertAlmostEqual(spot_to_tariff(interval_time, 'tasnetworks', 'TAS93', 100), 14.537, 2)


if __name__ == '__main__':
    unittest.main()



import unittest
from datetime import datetime
from zoneinfo import ZoneInfo
from aemo_to_tariff.endeavour import convert, convert_feed_in_tariff, time_zone

class TestEndeavour(unittest.TestCase):
    def test_convert_feed_in(self):
        interval_time = datetime(2023, 1, 15, 17, 0, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'N71'
        feed_in_price = convert_feed_in_tariff(interval_time, tariff_code, 100.0)
        self.assertAlmostEqual(feed_in_price, 10.00, places=1)
        
    def test_convert_N71_n61_feed_in(self):
        # 17:15	$55	Act	0.692 x 17.19¢	1189.88¢	0 x 31.98¢
        interval_time = datetime(2025, 11, 10, 17, 15, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'N61'
        feed_in_price = convert_feed_in_tariff(interval_time, tariff_code, 55.0)
        msg = f"Feed-in price for {tariff_code} at {interval_time} should be approximately 17.19"
        self.assertAlmostEqual(feed_in_price, 17.19 + 0.74, places=1, msg=msg)
        buyer_price = convert(interval_time, 'N71', 55.0)
        msg = f"Buyer price for {tariff_code} at {interval_time} should be approximately 31.98"
        self.assertAlmostEqual(buyer_price, 31.98 - 4.68, places=1, msg=msg)
        
        # 17:15	$55	Act	0.692 x 17.19¢	1189.88¢	0 x 31.98¢
        interval_time = datetime(2025, 11, 15, 17, 15, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'N61'
        feed_in_price = convert_feed_in_tariff(interval_time, tariff_code, 55.0)
        msg = f"Feed-in price for {tariff_code} at {interval_time} should be approximately 5.5"
        self.assertAlmostEqual(feed_in_price, 5.5, places=1, msg=msg)
        buyer_price = convert(interval_time, 'N71', 55.0)
        msg = f"Buyer price for {tariff_code} at {interval_time} should be approximately 31.98"
        self.assertAlmostEqual(buyer_price, 31.98 - 4.68, places=1, msg=msg)
        
        # 12:20	$-6	Act	0 x -0.67¢	0.00¢	0 x 4.55¢
        interval_time = datetime(2025, 11, 10, 12, 20, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'N61'
        feed_in_price = convert_feed_in_tariff(interval_time, tariff_code, -6.0)
        msg = f"Feed-in price for {tariff_code} at {interval_time} should be approximately -11.04"
        self.assertAlmostEqual(feed_in_price, -0.69 + 0.09, places=1, msg=msg)
        buyer_price = convert(interval_time, 'N71', -6.0)
        msg = f"Buyer price for {tariff_code} at {interval_time} should be approximately 12.30"
        self.assertAlmostEqual(buyer_price, 4.55 + 0.41, places=1, msg=msg)
        
        # 00:10	$121	Act	0 x 13.03¢	0.00¢	0 x 27.40¢
        interval_time = datetime(2025, 11, 10, 0, 10, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'N61'
        feed_in_price = convert_feed_in_tariff(interval_time, tariff_code, 121.0)
        msg = f"Feed-in price for {tariff_code} at {interval_time} should be approximately 0.00"
        self.assertAlmostEqual(feed_in_price, 13.03 + -2.9, places=1, msg=msg)
        
    def test_convert_high_season_peak(self):
        interval_time = datetime(2023, 1, 15, 17, 0, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'N71'
        rrp = 100.0
        expected_price = 31.7964
        price = convert(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(price, expected_price)

    def test_convert_low_season_peak(self):
        interval_time = datetime(2024, 8, 15, 17, 0, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'N71'
        rrp = 100.0
        expected_price = 23.8419
        price = convert(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(price, expected_price, places=4)

    def test_convert_off_peak(self):
        interval_time = datetime(2023, 7, 15, 10, 5, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'N71'
        rrp = 100.0
        expected_price = 15.97
        price = convert(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(price, expected_price, places=2)

    def test_convert_unknown_tariff(self):
        interval_time = datetime(2023, 7, 15, 10, 0, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'N999'
        rrp = 100.0
        with self.assertRaises(KeyError):
            convert(interval_time, tariff_code, rrp)
